/*
   Asm83.c:
   Source file to create asm83.dll
   Assembly Studio plug-in to output an Asm program to a .83p file

   Copyright (c) 2001 Ti-Calculator Programming Alliance
   Programmed for TI-86 by Jeremy Goetsch
   Converted to TI-83 by James Vernon <james@calc.org>
   http://tcpa.calc.org
   ICQ#: 71589304
*/

#include <stdio.h>
#include <string.h>

/* note that windows routines are used only for the About command */
#include <windows.h>
#include <winuser.h>

__declspec(dllexport) const char *GetPluginDescription()
{
	return("TI-83 program (.83p) file");
}

__declspec(dllexport) void About()
{
	MessageBox( NULL,
		"TI-83 program (.83p) file export\n"
		"Assembly Studio plugin\n"
        "Copyright (c) 2001 Ti-Calculator Programming Alliance\n"
        "Programmed for TI-86 by Jeremy Goetsch\n"
        "Converted to TI-83 by James Vernon\n"
        "james@calc.org <> http://tcpa.calc.org <> ICQ#:71589304",
        "About asm83.dll", MB_OK);
}

__declspec(dllexport) int SavePrgmFile(unsigned char *pData, unsigned nLength, const char *szFilename, char *szExt)
{
	FILE *out;
	unsigned i, wdval;
	unsigned checksum = 0;
	unsigned nCodeLength;
	char *p, pbuff[128];

	/* add extension to filename and open file */
	strcpy(pbuff, szFilename);
	strcat(pbuff, ".83p");
	if ((out = fopen(pbuff, "wb")) == NULL)
		return(0);

    /* header */
    fwrite("**TI83**\x1a\x0a\x00", 1, 11, out);
    /* comment */
	fwrite("Asm program created with Asmide8x v4.0\0\0\0\0", 1, 42, out);

	nCodeLength = nLength;
	wdval = nCodeLength + 17;
	fwrite(&wdval, 2, 1, out);	/* length to end of file */
	wdval = 0x000B;
	fwrite(&wdval, 2, 1, out);	/* always 0B,00 on the TI-83 */
	checksum += wdval;
	wdval = nCodeLength + 2;
	fwrite(&wdval, 2, 1, out);	/* length of code + checksum */
	checksum += (wdval & 0x00FF) + (wdval>>8);
	wdval = 0x06;
	fwrite(&wdval, 1, 1, out);	/* program type */
	checksum += wdval;

	/* get program name */
	if (!(p = strrchr(szFilename, '\\')))
		strncpy(pbuff, szFilename, 8);
	else
		strncpy(pbuff, p + 1, 8);
	pbuff[8] = '\0';

	wdval = strlen(pbuff);
    strupr(pbuff);
	fwrite(pbuff, 1, 8, out);
	for (i = 0; i < 8; i++)
		checksum += pbuff[i];

	wdval = nCodeLength + 2;
	fwrite(&wdval, 2, 1, out);	/* length of code + checksum */
	checksum += (wdval & 0x00FF) + (wdval>>8);
	wdval = nCodeLength;
	fwrite(&wdval, 2, 1, out);	/* length of code */
	checksum += (wdval & 0x00FF) + (wdval>>8);

	/* actual assembly code */
	fwrite(pData, 1, nLength, out);
	for (i = 0; i < nLength; i++)
		checksum += pData[i];
	fwrite(&checksum, 2, 1, out);

	/* return given extension */
	if (szExt != NULL)
		strcpy(szExt, ".83p");
	return(1);
}
